import rclpy
from rcl_interfaces.msg import Parameter
from ble_controller.utils import str_to_value
from ble_controller.base_controller import BaseController

class CliController(BaseController):
    def __init__(self):
        super().__init__("cli_controller")
    def cli_control(self):
        params=[]
        node_idx =-1
        while True:
            print("Press 1 to scan nodes")
            print("Press 2 to get parameters")
            print("Press 3 to change parameters")            
            print("Press 4 to get state")
            print("Press 5 to change state")
            print("Press 6 to exit")
            choice = input("Enter your choice: ")
            if choice == "1":
                self.find_nodes()
            elif choice == "2":
                for i in range(len(self.node_list)):
                    print(f'{i}: {self.node_list[i]}')
                node_idx = input("Enter node index: ")
                if int(node_idx) >= len(self.node_list) or int(node_idx) < 0:
                    print("Invalid node index")
                    continue
                params = self.node_get_parameters( self.node_list[int(node_idx)] )
                print(f'Got {len(params)} parameters')
            elif choice == "3":
                if len(params) == 0:
                    print("Please select a node first.")
                    continue
                for i in range(len(params)):
                    print(f'{i}: {params[i]}')
                param_idx = input("Enter parameter index: ")
                if int(param_idx) >= len(params) or int(param_idx) < 0:
                    print("Invalid parameter index")
                    continue
                new_param: Parameter = params[int(param_idx)]
                new_value = input(f"Enter new value for ({new_param.name}): ")
                new_param.value = str_to_value(new_param.value.type, new_value)
                self.node_change_parameter( self.node_list[int(node_idx)], [new_param] )
            elif choice == "4":
                for i in range(len(self.node_list)):
                    print(f'{i}: {self.node_list[i]}')
                node_idx = input("Enter node index: ")
                if int(node_idx) >= len(self.node_list) or int(node_idx) < 0:
                    print("Invalid node index")
                    continue
                print(self.node_get_state(self.node_list[int(node_idx)]))
            elif choice == "5":
                for i in range(len(self.node_list)):
                    print(f'{i}: {self.node_list[i]}')
                node_idx = input("Enter node index: ")
                if int(node_idx) >= len(self.node_list) or int(node_idx) < 0:
                    print("Invalid node index")
                    continue
                available_transitions = self.node_get_available_transitions(self.node_list[int(node_idx)])
                if available_transitions is None:
                    continue
                for i in range(len(available_transitions)):
                    print(f'{i}: {available_transitions[i].transition.label}')
                transition_idx = input("Enter transition index: ")
                if int(transition_idx) >= len(available_transitions) or int(transition_idx) < 0:
                    print("Invalid transition index")
                    continue
                self.node_change_state(self.node_list[int(node_idx)], available_transitions[int(transition_idx)].transition.id)
            elif choice == "6":
                break
            else:
                print("Invalid choice")
            print()
    
def main(args=None):
    rclpy.init(args=args)
    controllerer = CliController()
    try:
        controllerer.cli_control()
    except KeyboardInterrupt:
        print("KeyboardInterrupt")
    finally:
        controllerer.destroy_node()
        rclpy.shutdown()