
from enum import Enum
from rcl_interfaces.msg import ParameterType, ParameterValue
import ast
import rclpy
from rclpy.node import Node,Client
from typing import Any
from lifecycle_msgs.msg import State
from lifecycle_msgs.msg import Transition

class Type(Enum):
    NOT_SET = ParameterType.PARAMETER_NOT_SET
    BOOL = ParameterType.PARAMETER_BOOL
    INTEGER = ParameterType.PARAMETER_INTEGER
    DOUBLE = ParameterType.PARAMETER_DOUBLE
    STRING = ParameterType.PARAMETER_STRING
    BYTE_ARRAY = ParameterType.PARAMETER_BYTE_ARRAY
    BOOL_ARRAY = ParameterType.PARAMETER_BOOL_ARRAY
    INTEGER_ARRAY = ParameterType.PARAMETER_INTEGER_ARRAY
    DOUBLE_ARRAY = ParameterType.PARAMETER_DOUBLE_ARRAY
    STRING_ARRAY = ParameterType.PARAMETER_STRING_ARRAY
    
def extract_value(parameter_value: ParameterValue):
    type = Type(parameter_value.type)
    if type == Type.NOT_SET:
        return None
    elif type == Type.BOOL:
        return parameter_value.bool_value
    elif type == Type.INTEGER:
        return parameter_value.integer_value
    elif type == Type.DOUBLE:
        return parameter_value.double_value
    elif type == Type.STRING:
        return parameter_value.string_value
    elif type == Type.BYTE_ARRAY:
        return parameter_value.byte_array_value
    elif type == Type.BOOL_ARRAY:
        return parameter_value.bool_array_value
    elif type == Type.INTEGER_ARRAY:
        return parameter_value.integer_array_value
    elif type == Type.DOUBLE_ARRAY:
        return parameter_value.double_array_value
    elif type == Type.STRING_ARRAY:
        return parameter_value.string_array_value
    else:
        raise TypeError('The given value is not one of the allowed types.')
    
def str_to_value(type: int, value: str) -> ParameterValue:
    if type == Type.NOT_SET.value:
        return ParameterValue(type=type)
    elif type == Type.BOOL.value:
        return ParameterValue(type=type, bool_value=ast.literal_eval(value))
    elif type == Type.INTEGER.value:
        return ParameterValue(type=type, integer_value=int(value))
    elif type == Type.DOUBLE.value:
        return ParameterValue(type=type, double_value= float(value))
    elif type == Type.STRING.value:
        return ParameterValue(type=type, string_value=value)
    elif type == Type.BYTE_ARRAY.value:
        return ParameterValue(type=type, byte_array_value=bytes(value, 'utf-8'))
    elif type == Type.BOOL_ARRAY.value:
        return ParameterValue(type=type, bool_array_value=ast.literal_eval(value))
    elif type == Type.INTEGER_ARRAY.value:
        return ParameterValue(type=type, integer_array_value=ast.literal_eval(value))
    elif type == Type.DOUBLE_ARRAY.value:
        return ParameterValue(type=type, double_array_value=ast.literal_eval(value))
    elif type == Type.STRING_ARRAY.value:
        return ParameterValue(type=type, string_array_value=ast.literal_eval(value))
    else:
        raise TypeError('The given value is not one of the allowed types.')
    
    
def str_to_value2(type: int, value: str) -> ParameterValue:
    if type == Type.NOT_SET.value:
        return ParameterValue(type=type)
    elif type == Type.BOOL.value:
        return ParameterValue(type=type, bool_value=value)
    elif type == Type.INTEGER.value:
        return ParameterValue(type=type, integer_value=int(value))
    elif type == Type.DOUBLE.value:
        return ParameterValue(type=type, double_value= float(value))
    elif type == Type.STRING.value:
        return ParameterValue(type=type, string_value=value)
    elif type == Type.BYTE_ARRAY.value:
        return ParameterValue(type=type, byte_array_value=bytes(value, 'utf-8'))
    elif type == Type.BOOL_ARRAY.value:
        return ParameterValue(type=type, bool_array_value=ast.literal_eval(value))
    elif type == Type.INTEGER_ARRAY.value:
        return ParameterValue(type=type, integer_array_value=ast.literal_eval(value))
    elif type == Type.DOUBLE_ARRAY.value:
        return ParameterValue(type=type, double_array_value=ast.literal_eval(value))
    elif type == Type.STRING_ARRAY.value:
        return ParameterValue(type=type, string_array_value=ast.literal_eval(value))
    else:
        raise TypeError('The given value is not one of the allowed types.')
    
def get_label(msg_type, id):
    for key, value in vars(msg_type).items():
        if(value == id):
            return key
    return None

def transition_cls( transition):
    return Transition(
        id=transition,
        # label=get_label(Transition, transition)
    )
    
def state_cls(state):
    return State(
        id=state,
        label=get_label(State, state)
    )
    
def cli_call(node:Node, cli: Client, req: Any, timeout: float) -> Any:
    """Call a service client with a request."""
    while not cli.wait_for_service(timeout_sec=timeout):
        raise RuntimeError(f"Service {cli} not available")
    future = cli.call_async(req)
    rclpy.spin_until_future_complete(node, future, timeout_sec=timeout)
    cli = None
    if future.result() is not None:
        return future.result()
    else:
        raise RuntimeError(f"Service {cli.srv_name} call failed")